@@ -2,17 +2,19 @@
     BangPatterns,
     CPP,
     DefaultSignatures,
+    DataKinds,
+    PolyKinds,
     FlexibleContexts,
     FlexibleInstances,
     KindSignatures,
     MultiParamTypeClasses,
-    OverlappingInstances,
     OverloadedStrings,
     Rank2Types,
     ScopedTypeVariables,
     TypeOperators,
     UndecidableInstances
     #-}
+
 module Data.Csv.Conversion
     (
     -- * Type conversion
@@ -1018,7 +1020,7 @@ type Success a f r = a -> f r
 -- lets you compose several field conversions together in such a way
 -- that if any of them fail, the whole record conversion fails.
 newtype Parser a = Parser {
-      unParser :: forall f r.
+      unParser :: forall (f :: * -> *) (r :: *).
                   Failure f r
                -> Success a f r
                -> f r
@@ -1028,7 +1030,9 @@ instance Monad Parser where
     m >>= g = Parser $ \kf ks -> let ks' a = unParser (g a) kf ks
                                  in unParser m kf ks'
     {-# INLINE (>>=) #-}
-    return a = Parser $ \_kf ks -> ks a
+    (>>) = (*>)
+    {-# INLINE (>>) #-}
+    return = pure
     {-# INLINE return #-}
     fail msg = Parser $ \kf _ks -> kf msg
     {-# INLINE fail #-}
@@ -1039,7 +1043,7 @@ instance Functor Parser where
     {-# INLINE fmap #-}
 
 instance Applicative Parser where
-    pure  = return
+    pure a = Parser $ \_kf ks -> ks a
     {-# INLINE pure #-}
     (<*>) = apP
     {-# INLINE (<*>) #-}
@@ -1067,7 +1071,7 @@ apP :: Parser (a -> b) -> Parser a -> Parser b
 apP d e = do
   b <- d
   a <- e
-  return (b a)
+  pure (b a)
 {-# INLINE apP #-}
 
 -- | Run a 'Parser', returning either @'Left' errMsg@ or @'Right'
@@ -1200,7 +1204,12 @@ instance GToNamedRecordHeader a => GToNamedRecordHeader (M1 C c a)
 
 -- | Instance to ensure that you cannot derive DefaultOrdered for
 -- constructors without selectors.
+#if MIN_VERSION_base(4,9,0)
+instance DefaultOrdered (M1 S ('MetaSel 'Nothing srcpk srcstr decstr) a ())
+         => GToNamedRecordHeader (M1 S ('MetaSel 'Nothing srcpk srcstr decstr) a)
+#else
 instance DefaultOrdered (M1 S NoSelector a ()) => GToNamedRecordHeader (M1 S NoSelector a)
+#endif
   where
     gtoNamedRecordHeader _ =
         error "You cannot derive DefaultOrdered for constructors without selectors."
